﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace SerialUHF
{
    public partial class Form1 : Form
    {
        private string tString = string.Empty;
        private char _terminator = '\r';

        public Form1()
        {
            InitializeComponent();
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            GetAllPorts();
        }
        private void Form1_FormClosing(object sender, FormClosingEventArgs e)
        {
            serialPort1.Close();
        }

        public List<string> GetAllPorts()
        {
            List<String> allPorts = new List<String>();
            foreach (String portName in System.IO.Ports.SerialPort.GetPortNames())
            {
                allPorts.Add(portName);
                comboBoxComm.Items.Add(portName);
            }
            return allPorts;
        }

        private void serialPort1_DataReceived(object sender, System.IO.Ports.SerialDataReceivedEventArgs e)
        {
            //Initialize a buffer to hold the received data 
            byte[] buffer = new byte[serialPort1.ReadBufferSize];

            //There is no accurate method for checking how many bytes are read 
            //unless you check the return from the Read method 
            int bytesRead = serialPort1.Read(buffer, 0, buffer.Length);

            //For the example assume the data we are received is ASCII data. 
            tString += Encoding.ASCII.GetString(buffer, 0, bytesRead);
            //Check if string contains the terminator  
            while (tString.IndexOf((char)_terminator) > -1)
            {
                //If tString does contain terminator we cannot assume that it is the last character received 
                string workingString = tString.Substring(0, tString.IndexOf((char)_terminator)+1);
                //Remove the data up to the terminator from tString 
                tString = tString.Substring(tString.IndexOf((char)_terminator)+1);
                //Do something with workingString 
                transferText(workingString);
            }
        }

        public void transferText(string text)
        {
            if (textBoxSerialIn.InvokeRequired)
            {
                textBoxSerialIn.Invoke((MethodInvoker)delegate()
                {
                    transferText(text);
                });
            }
            else
            {
                if (text.IndexOf('\r')> -1)
                {
                    //text = text.Substring(0, text.IndexOf('\r'));
                    if ((text.IndexOf('\r') + 2) > text.Length)
                    {
                        text += "\n";
                    }
                    else text = text.Insert(text.IndexOf('\r')+2, "\n");
                    
                }
                
                if (text.IndexOf('\n') > -1)
                {
                    text = text.Insert(text.IndexOf('\n'), "\r");
                }
                // textBoxSerialIn.Text += text;
                // textBoxSerialIn.Text += "\n\r";
                textBoxSerialIn.AppendText(text);
            }
        }

        private void textBoxSerialIn_KeyPress(object sender, System.Windows.Forms.KeyPressEventArgs e)
        {
            string OutputData;
            string str;
            if (serialPort1.IsOpen)
            {
                OutputData = e.KeyChar.ToString();
                
                serialPort1.Write(OutputData);
            }
            else
            {
                MessageBox.Show("Port Not Open");
                e.Handled = true;
                return;
            }
        }

        private void buttonConnect_Click(object sender, EventArgs e)
        {
            if (comboBoxComm.Text == "")
            {
                MessageBox.Show("Please select port");
                return;
            }

            if (buttonConnect.Text == "Connect")
            {
                serialPort1.PortName = comboBoxComm.Text;
                serialPort1.ReceivedBytesThreshold = 1;
                serialPort1.BaudRate = int.Parse(comboBoxBaud.Text);
                serialPort1.DataBits = int.Parse("8");
                serialPort1.Parity = System.IO.Ports.Parity.None;
                serialPort1.StopBits = System.IO.Ports.StopBits.One;
                serialPort1.Handshake = System.IO.Ports.Handshake.None;
                try
                {
                    serialPort1.Open();
                }
                catch 
                {
                    MessageBox.Show("Failed to open port");
                    return;
                }
                buttonConnect.Text = "Disconnect";
            }
            else
            {
                serialPort1.Close();
                buttonConnect.Text = "Connect";

            }

        }

        private void buttonD_Click(object sender, EventArgs e)
        {
            string OutputData;
            if (serialPort1.IsOpen)
            {
                OutputData = "[D]";
                textBoxSerialIn.Text += "\n\r";
                textBoxSerialIn.Text += OutputData;
                textBoxSerialIn.Text += "\n\r";
                serialPort1.Write(OutputData);
            }
            else
            {
                MessageBox.Show("Port Not Open");
                
                return;
            }
        }

        private void buttonS_Click(object sender, EventArgs e)
        {
            string OutputData;
            if (serialPort1.IsOpen)
            {
                OutputData = "[S]";
                textBoxSerialIn.Text += "\n\r";
                textBoxSerialIn.Text += OutputData;
                textBoxSerialIn.Text += "\n\r";
                serialPort1.Write(OutputData);
            }
            else
            {
                MessageBox.Show("Port Not Open");
                
                return;
            }
        }

        private void buttonP_Click(object sender, EventArgs e)
        {
            string OutputData;
            if (serialPort1.IsOpen)
            {
                OutputData = "[P]";
                textBoxSerialIn.Text += "\n\r";
                textBoxSerialIn.Text += OutputData;
                textBoxSerialIn.Text += "\n\r";
                serialPort1.Write(OutputData);
            }
            else
            {
                MessageBox.Show("Port Not Open");
                
                return;
            }
        }

        private void buttonT_Click(object sender, EventArgs e)
        {
            string OutputData;
            if (serialPort1.IsOpen)
            {
                OutputData = "[T]";
                textBoxSerialIn.Text += "\n\r";
                textBoxSerialIn.Text += OutputData;
                textBoxSerialIn.Text += "\n\r";
                serialPort1.Write(OutputData);
            }
            else
            {
                MessageBox.Show("Port Not Open");
                
                return;
            }
        }

        private void buttonW_Click(object sender, EventArgs e)
        {
            string OutputData;

            if (textBoxWrite.Text == "")
            {
                MessageBox.Show("Nothing to write");
            }

            if (serialPort1.IsOpen)
            {
                textBoxWrite.Text = textBoxWrite.Text.ToUpper();
                OutputData = "[W";
                OutputData += textBoxWrite.Text;
                OutputData += "]";
                textBoxSerialIn.Text += "\n\r";
                textBoxSerialIn.Text += OutputData;
                textBoxSerialIn.Text += "\n\r";
                serialPort1.Write(OutputData);
            }
            else
            {
                MessageBox.Show("Port Not Open");
                
                return;
            }
        }


     }
}
